@@ -162,7 +162,7 @@ int file_openfile(char *filename, int rwmode, FILE **diskfile)
     char mode[4];
 
 #if defined(unix) || defined(__unix__) || defined(__unix)
-    char tempname[512], *cptr, user[80];
+    char tempname[1024], *cptr, user[80];
     struct passwd *pwd;
     int ii = 0;
 
@@ -201,11 +201,17 @@ int file_openfile(char *filename, int rwmode, FILE **diskfile)
             cptr = getenv("HOME");
             if (cptr)
             {
+                 if (strlen(cptr) + strlen(filename+1) > 1023)
+		      return(FILE_NOT_OPENED); 
+
                  strcpy(tempname, cptr);
                  strcat(tempname, filename+1);
             }
             else
             {
+                 if (strlen(filename) > 1023)
+		      return(FILE_NOT_OPENED); 
+
                  strcpy(tempname, filename);
             }
         }
@@ -225,6 +231,9 @@ int file_openfile(char *filename, int rwmode, FILE **diskfile)
             pwd = getpwnam(user);
 
             /* copy user's home directory */
+            if (strlen(pwd->pw_dir) + strlen(cptr) > 1023)
+		      return(FILE_NOT_OPENED); 
+
             strcpy(tempname, pwd->pw_dir);
             strcat(tempname, cptr);
         }
@@ -249,6 +258,10 @@ int file_openfile(char *filename, int rwmode, FILE **diskfile)
            {
               if ((f1 = fopen(filename, "rb")) != 0) /* try opening READONLY */
               {
+
+                 if (strlen(filename) + 7 > 1023)
+		      return(FILE_NOT_OPENED); 
+
                  strcpy(tempname, filename);
                  strcat(tempname, ".TmxFil");
                  if ((f2 = fopen(tempname, "wb")) != 0) /* create temp file */
@@ -553,7 +566,7 @@ int file_compress_open(char *filename, int rwmode, int *hdl)
 */
 {
     FILE *indiskfile, *outdiskfile;
-    int status, clobber = 0;
+    int status;
     char *cptr;
 
     /* open the compressed disk file */
@@ -572,7 +585,6 @@ int file_compress_open(char *filename, int rwmode, int *hdl)
     if (*cptr == '!')
     {
         /* clobber any existing file with the same name */
-        clobber = 1;
         cptr++;
         remove(cptr);
     }
@@ -635,6 +647,9 @@ int file_is_compressed(char *filename) /* I - FITS file name          */
     /* Open file.  Try various suffix combinations */  
     if (file_openfile(filename, 0, &diskfile))
     {
+      if (strlen(filename) > FLEN_FILENAME - 1)
+          return(0);
+
       strcpy(tmpfilename,filename);
       strcat(filename,".gz");
       if (file_openfile(filename, 0, &diskfile))
@@ -732,12 +747,122 @@ int file_checkfile (char *urltype, char *infile, char *outfile)
         /* and copied to this newly created output file.  The original file */
         /* will be closed, and the copy will be opened by CFITSIO for     */
         /* subsequent processing (possibly with READWRITE access).        */
-        if (strlen(outfile))
-            strcpy(file_outfile,outfile);
+        if (strlen(outfile)) {
+	    file_outfile[0] = '\0';
+            strncat(file_outfile,outfile,FLEN_FILENAME-1);
+        }
     }
 
     return 0;
 }
+/**********************************************************************/
+/**********************************************************************/
+/**********************************************************************/
+
+/****  driver routines for stream//: device (stdin or stdout)  ********/
+
+
+/*--------------------------------------------------------------------------*/
+int stream_open(char *filename, int rwmode, int *handle)
+{
+    /*
+        read from stdin
+    */
+    rwmode = (int) filename;  /* suppress unused parameter compiler warning */
+    *handle = 1;     /*  1 = stdin */   
+
+    return(0);
+}
+/*--------------------------------------------------------------------------*/
+int stream_create(char *filename, int *handle)
+{
+    /*
+        write to stdout
+    */
+
+    *handle = (int) filename;  /* suppress unused parameter compiler warning */
+    *handle = 2;         /*  2 = stdout */       
+    return(0);
+}
+/*--------------------------------------------------------------------------*/
+int stream_size(int handle, LONGLONG *filesize)
+/*
+  return the size of the file in bytes
+*/
+{
+    handle = 0;  /* suppress unused parameter compiler warning */
+    
+    /* this operation is not supported in a stream; return large value */
+    *filesize = LONG_MAX;
+    return(0);
+}
+/*--------------------------------------------------------------------------*/
+int stream_close(int handle)
+/*
+     don't have to close stdin or stdout 
+*/
+{
+    handle = 0;  /* suppress unused parameter compiler warning */
+    
+    return(0);
+}
+/*--------------------------------------------------------------------------*/
+int stream_flush(int handle)
+/*
+  flush the file
+*/
+{
+    if (handle == 2)
+       fflush(stdout);  
+
+    return(0);
+}
+/*--------------------------------------------------------------------------*/
+int stream_seek(int handle, LONGLONG offset)
+   /* 
+      seeking is not allowed in a stream
+   */
+{
+    offset = handle;  /* suppress unused parameter compiler warning */
+    return(1);
+}
+/*--------------------------------------------------------------------------*/
+int stream_read(int hdl, void *buffer, long nbytes)
+/*
+     reading from stdin stream 
+*/
+
+{
+    long nread;
+    
+    if (hdl != 1)
+       return(1);  /* can only read from stdin */
+
+    nread = (long) fread(buffer, 1, nbytes, stdin);
+
+    if (nread != nbytes)
+    {
+/*        return(READ_ERROR); */
+        return(END_OF_FILE);
+    }
+
+    return(0);
+}
+/*--------------------------------------------------------------------------*/
+int stream_write(int hdl, void *buffer, long nbytes)
+/*
+  write bytes at the current position in the file
+*/
+{
+    if (hdl != 2)
+       return(1);  /* can only write to stdout */
+
+    if((long) fwrite(buffer, 1, nbytes, stdout) != nbytes)
+        return(WRITE_ERROR);
+
+    return(0);
+}
+
 
 
 
