@@ -119,18 +119,6 @@
 
    Once the file is closed then the socket is closed.
 
-$Id: drvrnet.c,v 3.48 2007/08/27 18:47:42 pence Exp $
-
-$Log: drvrnet.c,v $
-Revision 3.48  2007/08/27 18:47:42  pence
-new version 3.060 of CFITSIO;  includes better support for tile-compressed
-images and for the full set of WCS keywords in binary tables.
-
-Revision 3.45  2005/12/21 18:18:01  pence
-New beta 3.005 release.  Contains new cfortran.h to support integer*8
-parameters when calling cfitsio from Fortran tasks.  Also has modified
-fitsio.h file that now assumes that the 'long long' data type is supported
-by the C compiler (which may not be the case for older compilers).
 
 Revision 1.56  2000/01/04 11:58:31  oneel
 Updates so that compressed network files are dealt with regardless of
@@ -243,6 +231,7 @@ static int ftp_open_network(char *url, FILE **ftpfile, FILE **command,
 static int root_send_buffer(int sock, int op, char *buffer, int buflen);
 static int root_recv_buffer(int sock, int *op, char *buffer,int buflen);
 static int root_openfile(char *filename, char *rwmode, int *sock);
+static int encode64(unsigned s_len, char *src, unsigned d_len, char *dst);
 
 /***************************/
 /* Static variables */
@@ -310,8 +299,15 @@ int http_open(char *filename, int rwmode, int *handle)
       goto error;
     } 
   } else {
+  
+    if (strlen(filename) >= MAXLEN - 4) {
+	  ffpmsg("http file name is too long (http_open)");
+          ffpmsg(filename);
+	  goto error;
+    }
+  
     alarm(NETTIMEOUT);
-    /* Try the .gz one */
+    /* Try the .gz one */    
     strcpy(newfilename,filename);
     strcat(newfilename,".gz");
     
@@ -755,9 +751,11 @@ static int http_open_network(char *url, FILE **httpfile, char *contentencoding,
   char recbuf[MAXLEN];
   char tmpstr[MAXLEN];
   char tmpstr1[SHORTLEN];
+  char tmpstr2[MAXLEN];
   char errorstr[MAXLEN];
   char proto[SHORTLEN];
   char host[SHORTLEN];
+  char userpass[MAXLEN];
   char fn[MAXLEN];
   char turl[MAXLEN];
   char *scratchstr;
@@ -772,13 +770,20 @@ static int http_open_network(char *url, FILE **httpfile, char *contentencoding,
 
   /* Parse the URL apart again */
   strcpy(turl,"http://");
-  strcat(turl,url);
+  strncat(turl,url,MAXLEN - 8);
   if (NET_ParseUrl(turl,proto,host,&port,fn)) {
     sprintf(errorstr,"URL Parse Error (http_open) %s",url);
     ffpmsg(errorstr);
     return (FILE_NOT_OPENED);
   }
 
+  /* Do we have a user:password combo ? */
+    strcpy(userpass, url);
+  if ((scratchstr = strchr(userpass, '@')) != NULL) {
+    *scratchstr = '\0';
+  } else
+    strcpy(userpass, "");
+
   /* Ph. Prugniel 2003/04/03
      Are we using a proxy?
      
@@ -825,11 +830,29 @@ static int http_open_network(char *url, FILE **httpfile, char *contentencoding,
   else
     sprintf(tmpstr,"GET %s HTTP/1.0\r\n",fn);
 
+  if (strcmp(userpass, "")) {
+    encode64(strlen(userpass), userpass, MAXLEN, tmpstr2);
+    sprintf(tmpstr1, "Authorization: Basic %s\r\n", tmpstr2);
+
+    if (strlen(tmpstr) + strlen(tmpstr1) > MAXLEN - 1)
+        return (FILE_NOT_OPENED);
+
+    strcat(tmpstr,tmpstr1);
+  }
+
   sprintf(tmpstr1,"User-Agent: HEASARC/CFITSIO/%-8.3f\r\n",ffvers(&version));
+
+  if (strlen(tmpstr) + strlen(tmpstr1) > MAXLEN - 1)
+        return (FILE_NOT_OPENED);
+
   strcat(tmpstr,tmpstr1);
 
   /* HTTP 1.1 servers require the following 'Host: ' string */
   sprintf(tmpstr1,"Host: %s:%-d\r\n\r\n",host,port);
+
+  if (strlen(tmpstr) + strlen(tmpstr1) > MAXLEN - 1)
+        return (FILE_NOT_OPENED);
+
   strcat(tmpstr,tmpstr1);
 
   status = NET_SendRaw(sock,tmpstr,strlen(tmpstr),NET_DEFAULT);
@@ -953,6 +976,12 @@ int ftp_open(char *filename, int rwmode, int *handle)
   /* Open the ftp connetion.  ftpfile is connected to the file port, 
      command is connected to port 21.  sock is the socket on port 21 */
 
+  if (strlen(filename) > MAXLEN - 4) {
+      ffpmsg("filename too long (ftp_open)");
+      ffpmsg(filename);
+      goto error;
+  } 
+
   alarm(NETTIMEOUT);
   strcpy(newfilename,filename);
   /* Does the file have a .Z or .gz in it */
@@ -1423,6 +1452,11 @@ int ftp_open_network(char *filename, FILE **ftpfile, FILE **command, int *sock)
   int port;
 
   /* parse the URL */
+  if (strlen(filename) > MAXLEN - 7) {
+    ffpmsg("ftp filename is too long (ftp_open)");
+    return (FILE_NOT_OPENED);
+  }
+
   strcpy(turl,"ftp://");
   strcat(turl,filename);
   if (NET_ParseUrl(turl,proto,host,&port,fn)) {
@@ -1838,6 +1872,11 @@ static int NET_ParseUrl(const char *url, char *proto, char *host, int *port,
   }
   /* do this only if http */
   if (!strcmp(proto,"http:")) {
+
+    /* Move past any user:password */
+    if ((thost = strchr(urlcopy, '@')) != NULL)
+      urlcopy = thost+1;
+
     strcpy(host,urlcopy);
     thost = host;
     while (*urlcopy != '/' && *urlcopy != ':' && *urlcopy) {
@@ -2645,4 +2684,58 @@ static int root_recv_buffer(int sock, int *op, char *buffer, int buflen)
   return recv1;
 
 }
+
+/*****************************************************************************/
+/*
+  Encode a string into MIME Base64 format string
+*/
+
+
+static int encode64(unsigned s_len, char *src, unsigned d_len, char *dst) {
+
+  static char base64[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+"abcdefghijklmnopqrstuvwxyz"
+"0123456789"
+"+/";
+
+  unsigned triad;
+
+
+  for (triad = 0; triad < s_len; triad += 3) {
+    unsigned long int sr;
+    unsigned byte;
+
+    for (byte = 0; (byte<3) && (triad+byte<s_len); ++byte) {
+      sr <<= 8;
+      sr |= (*(src+triad+byte) & 0xff);
+    }
+
+    /* shift left to next 6 bit alignment*/
+    sr <<= (6-((8*byte)%6))%6;
+
+    if (d_len < 4)
+      return 1;
+
+    *(dst+0) = *(dst+1) = *(dst+2) = *(dst+3) = '=';
+    switch(byte) {
+    case 3:
+      *(dst+3) = base64[sr&0x3f];
+      sr >>= 6;
+    case 2:
+      *(dst+2) = base64[sr&0x3f];
+      sr >>= 6;
+    case 1:
+      *(dst+1) = base64[sr&0x3f];
+      sr >>= 6;
+      *(dst+0) = base64[sr&0x3f];
+    }
+    dst += 4;
+    d_len -= 4;
+  }
+
+  *dst = '\0';
+  return 0;
+}
+
+
 #endif
